@@ -23,30 +23,20 @@
 #include <QImageReader>
 #include <QByteArray>
 #include <QBuffer>
+#include <QPainter>
 
 #include <kis_dom_utils.h>
 
 KisPngBrush::KisPngBrush(const QString& filename)
-    : KisScalingSizeBrush(filename)
+    : KisColorfulBrush(filename)
 {
     setBrushType(INVALID);
     setSpacing(0.25);
-    setHasColor(false);
 }
 
 KisPngBrush::KisPngBrush(const KisPngBrush &rhs)
-    : KisScalingSizeBrush(rhs)
+    : KisColorfulBrush(rhs)
 {
-    setSpacing(rhs.spacing());
-    if (brushTipImage().isGrayscale()) {
-        setBrushType(MASK);
-        setHasColor(false);
-    }
-    else {
-        setBrushType(IMAGE);
-        setHasColor(true);
-    }
-    setValid(rhs.valid());
 }
 
 KisBrush* KisPngBrush::clone() const
@@ -93,7 +83,7 @@ bool KisPngBrush::loadFromDevice(QIODevice *dev)
     }
     else {
         QFileInfo info(filename());
-        setName(info.baseName());
+        setName(info.completeBaseName());
     }
 
     QImage image = reader.read();
@@ -104,16 +94,47 @@ bool KisPngBrush::loadFromDevice(QIODevice *dev)
         return false;
     }
 
-    setBrushTipImage(image);
     setValid(true);
 
-    if (brushTipImage().isGrayscale()) {
+    bool hasAlpha = false;
+    for (int y = 0; y < image.height(); y++) {
+        for (int x = 0; x < image.width(); x++) {
+            if (qAlpha(image.pixel(x, y)) != 255) {
+                hasAlpha = true;
+                break;
+            }
+        }
+    }
+
+    const bool isAllGray = image.allGray();
+
+    if (isAllGray && !hasAlpha) {
+        // Make sure brush tips all have a white background
+        // NOTE: drawing it over white background can probably be skipped now...
+        //       Any images with an Alpha channel should be loaded as RGBA so
+        //       they can have the lightness and gradient options available
+        QImage base(image.size(), image.format());
+        if ((int)base.format() < (int)QImage::Format_RGB32) {
+            base = base.convertToFormat(QImage::Format_ARGB32);
+        }
+        QPainter gc(&base);
+        gc.fillRect(base.rect(), Qt::white);
+        gc.drawImage(0, 0, image);
+        gc.end();
+        QImage converted = base.convertToFormat(QImage::Format_Grayscale8);
+        setBrushTipImage(converted);
         setBrushType(MASK);
-        setHasColor(false);
+        setBrushApplication(ALPHAMASK);
+        setHasColorAndTransparency(false);
     }
     else {
+        if ((int)image.format() < (int)QImage::Format_RGB32) {
+            image = image.convertToFormat(QImage::Format_ARGB32);
+        }
+        setBrushTipImage(image);
         setBrushType(IMAGE);
-        setHasColor(true);
+        setBrushApplication(isAllGray ? ALPHAMASK : IMAGESTAMP);
+        setHasColorAndTransparency(!isAllGray);
     }
 
     setWidth(brushTipImage().width());
@@ -149,5 +170,5 @@ QString KisPngBrush::defaultFileExtension() const
 void KisPngBrush::toXML(QDomDocument& d, QDomElement& e) const
 {
     predefinedBrushToXML("png_brush", e);
-    KisBrush::toXML(d, e);
+    KisColorfulBrush::toXML(d, e);
 }
