@@ -59,7 +59,6 @@
 #include "kis_command_utils.h"
 #include "kis_pointer_utils.h"
 
-
 #include <KoIcon.h>
 
 #include <QMenu>
@@ -67,6 +66,7 @@
 #include <FlakeDebug.h>
 #include <klocalizedstring.h>
 #include <QPainter>
+#include <QPainterPath>
 #include <QBitmap>
 #include <QTabWidget>
 
@@ -108,82 +108,44 @@ struct KoPathTool::PathSegment {
 };
 
 KoPathTool::KoPathTool(KoCanvasBase *canvas)
-        : KoToolBase(canvas)
-        , m_pointSelection(this)
-        , m_activeHandle(0)
-        , m_handleRadius(3)
-        , m_activeSegment(0)
-        , m_currentStrategy(0)
-        , m_activatedTemporarily(false)
+    : KoToolBase(canvas)
+    , m_pointSelection(this)
+    , m_activatedTemporarily(false)
 {
-    QActionGroup *points = new QActionGroup(this);
+    m_points = new QActionGroup(this);
     // m_pointTypeGroup->setExclusive(true);
-    KisActionRegistry *actionRegistry = KisActionRegistry::instance();
-    m_actionPathPointCorner = actionRegistry->makeQAction("pathpoint-corner", this);
-    addAction("pathpoint-corner", m_actionPathPointCorner);
-    m_actionPathPointCorner->setData(KoPathPointTypeCommand::Corner);
-    points->addAction(m_actionPathPointCorner);
 
-    m_actionPathPointSmooth = actionRegistry->makeQAction("pathpoint-smooth", this);
-    addAction("pathpoint-smooth", m_actionPathPointSmooth);
-    m_actionPathPointSmooth->setData(KoPathPointTypeCommand::Smooth);
-    points->addAction(m_actionPathPointSmooth);
+    m_actionPathPointCorner = action("pathpoint-corner");
+    if (m_actionPathPointCorner) {
+        m_actionPathPointCorner->setData(KoPathPointTypeCommand::Corner);
+        m_points->addAction(m_actionPathPointCorner);
+    }
+    m_actionPathPointSmooth = action("pathpoint-smooth");
+    if (m_actionPathPointSmooth) {
+        m_actionPathPointSmooth->setData(KoPathPointTypeCommand::Smooth);
+        m_points->addAction(m_actionPathPointSmooth);
+    }
 
-    m_actionPathPointSymmetric = actionRegistry->makeQAction("pathpoint-symmetric", this);
-    addAction("pathpoint-symmetric", m_actionPathPointSymmetric);
-    m_actionPathPointSymmetric->setData(KoPathPointTypeCommand::Symmetric);
-    points->addAction(m_actionPathPointSymmetric);
+    m_actionPathPointSymmetric = action("pathpoint-symmetric");
+    if (m_actionPathPointSymmetric) {
+        m_actionPathPointSymmetric->setData(KoPathPointTypeCommand::Symmetric);
+        m_points->addAction(m_actionPathPointSymmetric);
+    }
 
-    m_actionCurvePoint = actionRegistry->makeQAction("pathpoint-curve", this);
-    addAction("pathpoint-curve", m_actionCurvePoint);
-    connect(m_actionCurvePoint, SIGNAL(triggered()), this, SLOT(pointToCurve()));
-
-    m_actionLinePoint = actionRegistry->makeQAction("pathpoint-line", this);
-    addAction("pathpoint-line", m_actionLinePoint);
-    connect(m_actionLinePoint, SIGNAL(triggered()), this, SLOT(pointToLine()));
-
-    m_actionLineSegment = actionRegistry->makeQAction("pathsegment-line", this);
-    addAction("pathsegment-line", m_actionLineSegment);
-    connect(m_actionLineSegment, SIGNAL(triggered()), this, SLOT(segmentToLine()));
-
-    m_actionCurveSegment = actionRegistry->makeQAction("pathsegment-curve", this);
-    addAction("pathsegment-curve", m_actionCurveSegment);
-    connect(m_actionCurveSegment, SIGNAL(triggered()), this, SLOT(segmentToCurve()));
-
-    m_actionAddPoint = actionRegistry->makeQAction("pathpoint-insert", this);
-    addAction("pathpoint-insert", m_actionAddPoint);
-    connect(m_actionAddPoint, SIGNAL(triggered()), this, SLOT(insertPoints()));
-
-    m_actionRemovePoint = actionRegistry->makeQAction("pathpoint-remove", this);
-    addAction("pathpoint-remove", m_actionRemovePoint);
-    connect(m_actionRemovePoint, SIGNAL(triggered()), this, SLOT(removePoints()));
-
-    m_actionBreakPoint = actionRegistry->makeQAction("path-break-point", this);
-    addAction("path-break-point", m_actionBreakPoint);
-    connect(m_actionBreakPoint, SIGNAL(triggered()), this, SLOT(breakAtPoint()));
-
-    m_actionBreakSegment = actionRegistry->makeQAction("path-break-segment", this);
-    addAction("path-break-segment", m_actionBreakSegment);
-    connect(m_actionBreakSegment, SIGNAL(triggered()), this, SLOT(breakAtSegment()));
-
-    m_actionJoinSegment = actionRegistry->makeQAction("pathpoint-join", this);
-    addAction("pathpoint-join", m_actionJoinSegment);
-    connect(m_actionJoinSegment, SIGNAL(triggered()), this, SLOT(joinPoints()));
-
-    m_actionMergePoints = actionRegistry->makeQAction("pathpoint-merge", this);
-    addAction("pathpoint-merge", m_actionMergePoints);
-    connect(m_actionMergePoints, SIGNAL(triggered()), this, SLOT(mergePoints()));
-
-    m_actionConvertToPath = actionRegistry->makeQAction("convert-to-path", this);
-    addAction("convert-to-path", m_actionConvertToPath);
-    connect(m_actionConvertToPath, SIGNAL(triggered()), this, SLOT(convertToPath()));
+    m_actionCurvePoint = action("pathpoint-curve");
+    m_actionLinePoint = action("pathpoint-line");
+    m_actionLineSegment = action("pathsegment-line");
+    m_actionCurveSegment = action("pathsegment-curve");
+    m_actionAddPoint = action("pathpoint-insert");
+    m_actionRemovePoint = action("pathpoint-remove");
+    m_actionBreakPoint = action("path-break-point");
+    m_actionBreakSegment = action("path-break-segment");
+    m_actionJoinSegment = action("pathpoint-join");
+    m_actionMergePoints = action("pathpoint-merge");
+    m_actionConvertToPath = action("convert-to-path");
 
     m_contextMenu.reset(new QMenu());
 
-
-    connect(points, SIGNAL(triggered(QAction*)), this, SLOT(pointTypeChanged(QAction*)));
-    connect(&m_pointSelection, SIGNAL(selectionChanged()), this, SLOT(pointSelectionChanged()));
-
     QBitmap b = QBitmap::fromData(QSize(16, 16), needle_bits);
     QBitmap m = b.createHeuristicMask(false);
 
@@ -193,13 +155,12 @@ KoPathTool::KoPathTool(KoCanvasBase *canvas)
     m = b.createHeuristicMask(false);
 
     m_moveCursor = QCursor(b, m, 2, 0);
+
+    connect(&m_pointSelection, SIGNAL(selectionChanged()), SLOT(repaintDecorations()));
 }
 
 KoPathTool::~KoPathTool()
 {
-    delete m_activeHandle;
-    delete m_activeSegment;
-    delete m_currentStrategy;
 }
 
 QList<QPointer<QWidget> >  KoPathTool::createOptionWidgets()
@@ -232,8 +193,8 @@ void KoPathTool::pointTypeChanged(QAction *type)
         }
 
         KUndo2Command *command =
-            new KoPathPointTypeCommand(selectedPoints,
-                                       static_cast<KoPathPointTypeCommand::PointType>(type->data().toInt()));
+                new KoPathPointTypeCommand(selectedPoints,
+                                           static_cast<KoPathPointTypeCommand::PointType>(type->data().toInt()));
 
         if (initialConversionCommand) {
             using namespace KisCommandUtils;
@@ -275,10 +236,9 @@ void KoPathTool::removePoints()
     Q_D(KoToolBase);
     if (m_pointSelection.size() > 0) {
         KUndo2Command *cmd = KoPathPointRemoveCommand::createCommand(m_pointSelection.selectedPointsData(), d->canvas->shapeController());
-        PointHandle *pointHandle = dynamic_cast<PointHandle*>(m_activeHandle);
+        PointHandle *pointHandle = dynamic_cast<PointHandle*>(m_activeHandle.data());
         if (pointHandle && m_pointSelection.contains(pointHandle->activePoint())) {
-            delete m_activeHandle;
-            m_activeHandle = 0;
+            m_activeHandle.reset();
         }
         clearActivePointSelectionReferences();
         d->canvas->addCommand(cmd);
@@ -391,7 +351,8 @@ void KoPathTool::convertToPath()
         const QList<KoShape*> oldSelectedShapes = implicitCastList<KoShape*>(textShapes);
 
 
-        new KoKeepShapesSelectedCommand(oldSelectedShapes, {}, canvas()->selectedShapesProxy(), false, cmd);
+        new KoKeepShapesSelectedCommand(oldSelectedShapes, {}, canvas()->selectedShapesProxy(),
+                                        KisCommandUtils::FlipFlopCommand::State::INITIALIZING, cmd);
 
         QList<KoShape*> newSelectedShapes;
         Q_FOREACH (KoSvgTextShape *shape, textShapes) {
@@ -413,7 +374,8 @@ void KoPathTool::convertToPath()
 
         canvas()->shapeController()->removeShapes(oldSelectedShapes, cmd);
 
-        new KoKeepShapesSelectedCommand({}, newSelectedShapes, canvas()->selectedShapesProxy(), true, cmd);
+        new KoKeepShapesSelectedCommand({}, newSelectedShapes, canvas()->selectedShapesProxy(),
+                                        KisCommandUtils::FlipFlopCommand::State::FINALIZING, cmd);
 
         canvas()->addCommand(cmd);
     }
@@ -511,7 +473,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 
     Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
         KisHandlePainterHelper helper =
-            KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                KoShape::createHandlePainterHelperView(&painter, shape, converter, handleRadius());
         helper.setHandleStyle(KisHandleStyle::primarySelection());
 
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
@@ -533,14 +495,13 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
         painter.restore();
     }
 
-    m_pointSelection.paint(painter, converter, m_handleRadius);
+    m_pointSelection.paint(painter, converter, handleRadius());
 
     if (m_activeHandle) {
         if (m_activeHandle->check(m_pointSelection.selectedShapes())) {
-            m_activeHandle->paint(painter, converter, m_handleRadius);
+            m_activeHandle->paint(painter, converter, handleRadius());
         } else {
-            delete m_activeHandle;
-            m_activeHandle = 0;
+            m_activeHandle.reset();
         }
     } else if (m_activeSegment && m_activeSegment->isValid()) {
 
@@ -551,8 +512,10 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
             KoPathPointIndex index = shape->pathPointIndex(m_activeSegment->segmentStart);
             KoPathSegment segment = shape->segmentByIndex(index).toCubic();
 
+            KIS_SAFE_ASSERT_RECOVER_RETURN(segment.isValid());
+
             KisHandlePainterHelper helper =
-                KoShape::createHandlePainterHelper(&painter, shape, converter, m_handleRadius);
+                    KoShape::createHandlePainterHelperView(&painter, shape, converter, handleRadius());
             helper.setHandleStyle(KisHandleStyle::secondarySelection());
 
             QPainterPath path;
@@ -569,19 +532,46 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 
     if (m_currentStrategy) {
         painter.save();
-        KoShape::applyConversion(painter, converter);
+        painter.setTransform(converter.documentToView(), true);
         d->canvas->snapGuide()->paint(painter, converter);
         painter.restore();
     }
 }
 
-void KoPathTool::repaintDecorations()
+QRectF KoPathTool::decorationsRect() const
 {
+    const_cast<KoPathToolSelection&>(m_pointSelection).update();
+
+    QRectF newDecorationsRect;
+
     Q_FOREACH (KoShape *shape, m_pointSelection.selectedShapes()) {
-        repaint(shape->boundingRect());
+        newDecorationsRect |= kisGrowRect(shape->boundingRect(), handleDocRadius());
     }
 
-    m_pointSelection.repaint();
+    Q_FOREACH(const KoPathPoint *point, m_pointSelection.selectedPoints()) {
+        newDecorationsRect |= kisGrowRect(point->boundingRect(false), handleDocRadius());
+    }
+
+    if (m_activeHandle) {
+        newDecorationsRect |= kisGrowRect(m_activeHandle->boundingRect(), handleDocRadius());
+    }
+
+    if (m_activeSegment) {
+        KoPathPointIndex index = m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart);
+        KoPathSegment segment = m_activeSegment->path->segmentByIndex(index);
+
+        QRectF rect = segment.boundingRect();
+        rect = m_activeSegment->path->shapeToDocument(rect);
+
+        newDecorationsRect |= kisGrowRect(rect, handleDocRadius());
+    }
+
+    return newDecorationsRect;
+}
+
+void KoPathTool::repaintDecorations()
+{
+    KoToolBase::repaintDecorations();
     updateOptionsWidget();
 }
 
@@ -590,7 +580,7 @@ void KoPathTool::mousePressEvent(KoPointerEvent *event)
     // we are moving if we hit a point and use the left mouse button
     event->ignore();
     if (m_activeHandle) {
-        m_currentStrategy = m_activeHandle->handleMousePress(event);
+        m_currentStrategy.reset(m_activeHandle->handleMousePress(event));
         event->accept();
     } else {
         if (event->button() & Qt::LeftButton) {
@@ -606,7 +596,7 @@ void KoPathTool::mousePressEvent(KoPointerEvent *event)
                 m_pointSelection.add(segment.second(), false);
 
                 KoPathPointData data(shape, index);
-                m_currentStrategy = new KoPathSegmentChangeStrategy(this, event->point, data, m_activeSegment->positionOnSegment);
+                m_currentStrategy.reset(new KoPathSegmentChangeStrategy(this, event->point, data, m_activeSegment->positionOnSegment));
                 event->accept();
             } else {
 
@@ -624,7 +614,7 @@ void KoPathTool::mousePressEvent(KoPointerEvent *event)
                     selection->select(shape);
                 } else {
                     KIS_ASSERT_RECOVER_RETURN(m_currentStrategy == 0);
-                    m_currentStrategy = new KoPathPointRubberSelectStrategy(this, event->point);
+                    m_currentStrategy.reset(new KoPathPointRubberSelectStrategy(this, event->point));
                     event->accept();
                 }
             }
@@ -641,26 +631,14 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
         m_lastPoint = event->point;
         m_currentStrategy->handleMouseMove(event->point, event->modifiers());
 
-        // repaint new handle positions
-        m_pointSelection.repaint();
-        if (m_activeHandle) {
-            m_activeHandle->repaint();
-        }
-
-        if (m_activeSegment) {
-            repaintSegment(m_activeSegment);
-        }
+        repaintDecorations();
 
         return;
     }
 
     if (m_activeSegment) {
-        KoPathPointIndex index = m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart);
-        KoPathSegment segment = m_activeSegment->path->segmentByIndex(index);
-        repaint(segment.boundingRect());
-
-        delete m_activeSegment;
-        m_activeSegment = 0;
+        m_activeSegment.reset();
+        repaintDecorations();
     }
 
     Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
@@ -671,19 +649,15 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
             if (handleId != -1) {
                 useCursor(m_moveCursor);
                 emit statusTextChanged(i18n("Drag to move handle."));
-                if (m_activeHandle)
-                    m_activeHandle->repaint();
-                delete m_activeHandle;
+                m_activeHandle.reset();
 
                 if (KoConnectionShape * connectionShape = dynamic_cast<KoConnectionShape*>(parameterShape)) {
-                    //qDebug() << "handleId" << handleId;
-                    m_activeHandle = new ConnectionHandle(this, connectionShape, handleId);
-                    m_activeHandle->repaint();
+                    m_activeHandle.reset(new ConnectionHandle(this, connectionShape, handleId));
+                    repaintDecorations();
                     return;
                 } else {
-                    //qDebug() << "handleId" << handleId;
-                    m_activeHandle = new ParameterHandle(this, parameterShape, handleId);
-                    m_activeHandle->repaint();
+                    m_activeHandle.reset(new ParameterHandle(this, parameterShape, handleId));
+                    repaintDecorations();
                     return;
                 }
             }
@@ -729,8 +703,10 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
                     }
                 }
 
-                if (! bestPoint)
+                if (! bestPoint) {
+                    repaintDecorations();
                     return;
+                }
 
                 useCursor(m_moveCursor);
                 if (bestPointType == KoPathPoint::Node)
@@ -738,15 +714,12 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
                 else
                     emit statusTextChanged(i18n("Drag to move control point."));
 
-                PointHandle *prev = dynamic_cast<PointHandle*>(m_activeHandle);
+                PointHandle *prev = dynamic_cast<PointHandle*>(m_activeHandle.data());
                 if (prev && prev->activePoint() == bestPoint && prev->activePointType() == bestPointType)
                     return; // no change;
 
-                if (m_activeHandle)
-                    m_activeHandle->repaint();
-                delete m_activeHandle;
-                m_activeHandle = new PointHandle(this, bestPoint, bestPointType);
-                m_activeHandle->repaint();
+                m_activeHandle.reset(new PointHandle(this, bestPoint, bestPointType));
+                repaintDecorations();
                 return;
             }
         }
@@ -755,18 +728,16 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
     useCursor(m_selectCursor);
 
     if (m_activeHandle) {
-        m_activeHandle->repaint();
+        m_activeHandle.reset();
+        repaintDecorations();
     }
 
-    delete m_activeHandle;
-    m_activeHandle = 0;
-
     PathSegment *hoveredSegment = segmentAtPoint(event->point);
     if(hoveredSegment) {
         useCursor(Qt::PointingHandCursor);
         emit statusTextChanged(i18n("Drag to change curve directly. Double click to insert new path point."));
-        m_activeSegment = hoveredSegment;
-        repaintSegment(m_activeSegment);
+        m_activeSegment.reset(hoveredSegment);
+        repaintDecorations();
     } else {
         uint selectedPointCount = m_pointSelection.size();
         if (selectedPointCount == 0)
@@ -778,15 +749,6 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
     }
 }
 
-void KoPathTool::repaintSegment(PathSegment *pathSegment)
-{
-    if (!pathSegment || !pathSegment->isValid()) return;
-
-    KoPathPointIndex index = pathSegment->path->pathPointIndex(pathSegment->segmentStart);
-    KoPathSegment segment = pathSegment->path->segmentByIndex(index);
-    repaint(segment.boundingRect());
-}
-
 void KoPathTool::mouseReleaseEvent(KoPointerEvent *event)
 {
     Q_D(KoToolBase);
@@ -796,19 +758,18 @@ void KoPathTool::mouseReleaseEvent(KoPointerEvent *event)
         KUndo2Command *command = m_currentStrategy->createCommand();
         if (command)
             d->canvas->addCommand(command);
-        if (hadNoSelection && dynamic_cast<KoPathPointRubberSelectStrategy*>(m_currentStrategy)
+        if (hadNoSelection && dynamic_cast<KoPathPointRubberSelectStrategy*>(m_currentStrategy.data())
                 && !m_pointSelection.hasSelection()) {
             // the click didn't do anything at all. Allow it to be used by others.
             event->ignore();
         }
-        delete m_currentStrategy;
-        m_currentStrategy = 0;
+        m_currentStrategy.reset();
+        repaintDecorations();
     }
 }
 
 void KoPathTool::keyPressEvent(QKeyEvent *event)
 {
-    Q_D(KoToolBase);
     if (m_currentStrategy) {
         switch (event->key()) {
         case Qt::Key_Control:
@@ -821,8 +782,7 @@ void KoPathTool::keyPressEvent(QKeyEvent *event)
             break;
         case Qt::Key_Escape:
             m_currentStrategy->cancelInteraction();
-            delete m_currentStrategy;
-            m_currentStrategy = 0;
+            m_currentStrategy.reset();
             break;
         default:
             event->ignore();
@@ -831,13 +791,13 @@ void KoPathTool::keyPressEvent(QKeyEvent *event)
     } else {
         switch (event->key()) {
 #ifndef NDEBUG
-        case Qt::Key_D:
-            if (m_pointSelection.objectCount() == 1) {
-                QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
-                KoPathShapePrivate *p = static_cast<KoPathShapePrivate*>(selectedPoints[0].pathShape->priv());
-                p->debugPath();
-            }
-            break;
+//        case Qt::Key_D:
+//            if (m_pointSelection.objectCount() == 1) {
+//                QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
+//                KoPathShapePrivate *p = static_cast<KoPathShapePrivate*>(selectedPoints[0].pathShape->priv());
+//                p->debugPath();
+//            }
+//            break;
 #endif
         case Qt::Key_B:
             if (m_pointSelection.size() == 1)
@@ -883,8 +843,8 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
     if (!m_activeHandle && m_activeSegment && m_activeSegment->isValid()) {
         QList<KoPathPointData> segments;
         segments.append(
-            KoPathPointData(m_activeSegment->path,
-                            m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart)));
+                    KoPathPointData(m_activeSegment->path,
+                                    m_activeSegment->path->pathPointIndex(m_activeSegment->segmentStart)));
 
         KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, m_activeSegment->positionOnSegment);
         d->canvas->addCommand(cmd);
@@ -909,8 +869,6 @@ void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
 
 KoPathTool::PathSegment* KoPathTool::segmentAtPoint(const QPointF &point)
 {
-    Q_D(KoToolBase);
-
     // the max allowed distance from a segment
     const QRectF grabRoi = handleGrabRect(point);
     const qreal distanceThreshold = 0.5 * KisAlgebra2D::maxDimension(grabRoi);
@@ -964,38 +922,60 @@ void KoPathTool::activate(ToolActivation activation, const QSet<KoShape*> &shape
 
     m_activatedTemporarily = activation == TemporaryActivation;
 
-    // retrieve the actual global handle radius
-    m_handleRadius = handleRadius();
     d->canvas->snapGuide()->reset();
 
     useCursor(m_selectCursor);
     m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
     m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionContentChanged()), this, SLOT(updateActions()));
+
+    m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionChanged()), this, SLOT(repaintDecorations()));
+    m_canvasConnections.addConnection(d->canvas->selectedShapesProxy(), SIGNAL(selectionContentChanged()), this, SLOT(repaintDecorations()));
     m_shapeFillResourceConnector.connectToCanvas(d->canvas);
 
     initializeWithShapes(shapes.toList());
+
+    connect(m_actionCurvePoint, SIGNAL(triggered()), this, SLOT(pointToCurve()), Qt::UniqueConnection);
+    connect(m_actionLinePoint, SIGNAL(triggered()), this, SLOT(pointToLine()), Qt::UniqueConnection);
+    connect(m_actionLineSegment, SIGNAL(triggered()), this, SLOT(segmentToLine()), Qt::UniqueConnection);
+    connect(m_actionCurveSegment, SIGNAL(triggered()), this, SLOT(segmentToCurve()), Qt::UniqueConnection);
+    connect(m_actionAddPoint, SIGNAL(triggered()), this, SLOT(insertPoints()), Qt::UniqueConnection);
+    connect(m_actionRemovePoint, SIGNAL(triggered()), this, SLOT(removePoints()), Qt::UniqueConnection);
+    connect(m_actionBreakPoint, SIGNAL(triggered()), this, SLOT(breakAtPoint()), Qt::UniqueConnection);
+    connect(m_actionBreakSegment, SIGNAL(triggered()), this, SLOT(breakAtSegment()), Qt::UniqueConnection);
+    connect(m_actionJoinSegment, SIGNAL(triggered()), this, SLOT(joinPoints()), Qt::UniqueConnection);
+    connect(m_actionMergePoints, SIGNAL(triggered()), this, SLOT(mergePoints()), Qt::UniqueConnection);
+    connect(m_actionConvertToPath, SIGNAL(triggered()), this, SLOT(convertToPath()), Qt::UniqueConnection);
+    connect(m_points, SIGNAL(triggered(QAction*)), this, SLOT(pointTypeChanged(QAction*)), Qt::UniqueConnection);
+    connect(&m_pointSelection, SIGNAL(selectionChanged()), this, SLOT(pointSelectionChanged()), Qt::UniqueConnection);
+
 }
 
 void KoPathTool::slotSelectionChanged()
 {
     Q_D(KoToolBase);
     QList<KoShape*> shapes =
-        d->canvas->selectedShapesProxy()->selection()->selectedEditableShapesAndDelegates();
+            d->canvas->selectedShapesProxy()->selection()->selectedEditableShapesAndDelegates();
 
     initializeWithShapes(shapes);
 }
 
-void KoPathTool::clearActivePointSelectionReferences()
+void KoPathTool::notifyPathPointsChanged(KoPathShape *shape)
 {
-    delete m_activeHandle;
-    m_activeHandle = 0;
-    delete m_activeSegment;
-    m_activeSegment = 0;
+    Q_UNUSED(shape);
 
-    m_pointSelection.clear();
+    // active handle and selection might have already become invalid, so just
+    // delete them without dereferencing anything...
+
+    m_activeHandle.reset();
+    m_activeSegment.reset();
 }
 
-#include "kis_pointer_utils.h"
+void KoPathTool::clearActivePointSelectionReferences()
+{
+    m_activeHandle.reset();
+    m_activeSegment.reset();
+    m_pointSelection.clear();
+}
 
 void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
 {
@@ -1009,7 +989,7 @@ void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
     }
 
     const QRectF oldBoundingRect =
-        KoShape::boundingRect(implicitCastList<KoShape*>(m_pointSelection.selectedShapes()));
+            KoShape::boundingRect(implicitCastList<KoShape*>(m_pointSelection.selectedShapes()));
 
     if (selectedShapes != m_pointSelection.selectedShapes()) {
         clearActivePointSelectionReferences();
@@ -1017,14 +997,6 @@ void KoPathTool::initializeWithShapes(const QList<KoShape*> shapes)
         repaintDecorations();
     }
 
-    Q_FOREACH (KoPathShape *shape, selectedShapes) {
-        // as the tool is just in activation repaintDecorations does not yet get called
-        // so we need to use repaint of the tool and it is only needed to repaint the
-        // current canvas
-        repaint(shape->boundingRect());
-    }
-    repaint(oldBoundingRect);
-
     updateOptionsWidget();
     updateActions();
 }
@@ -1036,7 +1008,7 @@ void KoPathTool::updateOptionsWidget()
     Q_FOREACH (KoPathShape *shape, selectedShapes) {
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
         type |= parameterShape && parameterShape->isParametricShape() ?
-                PathToolOptionWidget::ParametricShape : PathToolOptionWidget::PlainPath;
+                    PathToolOptionWidget::ParametricShape : PathToolOptionWidget::PlainPath;
     }
 
     emit singleShapeChanged(selectedShapes.size() == 1 ? selectedShapes.first() : 0);
@@ -1064,15 +1036,15 @@ void KoPathTool::updateActions()
             const int pointIndex = pd.pointIndex.second;
 
             canBreakAtPoint |= pd.pathShape->isClosedSubpath(subpathIndex) ||
-                (pointIndex > 0 && pointIndex < pd.pathShape->subpathPointCount(subpathIndex) - 1);
+                    (pointIndex > 0 && pointIndex < pd.pathShape->subpathPointCount(subpathIndex) - 1);
 
             KoPathPoint *point = pd.pathShape->pointByIndex(pd.pointIndex);
 
             hasNonSmoothPoints |= !(point->properties() & KoPathPoint::IsSmooth);
             hasNonSymmetricPoints |= !(point->properties() & KoPathPoint::IsSymmetric);
             hasNonSplitPoints |=
-                point->properties() & KoPathPoint::IsSymmetric ||
-                point->properties() & KoPathPoint::IsSmooth;
+                    point->properties() & KoPathPoint::IsSymmetric ||
+                    point->properties() & KoPathPoint::IsSmooth;
 
             hasNonLinePoints |= point->activeControlPoint1() || point->activeControlPoint2();
             hasNonCurvePoints |= !point->activeControlPoint1() && !point->activeControlPoint2();
@@ -1137,7 +1109,7 @@ void KoPathTool::updateActions()
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
         KoSvgTextShape *textShape = dynamic_cast<KoSvgTextShape*>(shape);
         if (textShape ||
-            (parameterShape && parameterShape->isParametricShape())) {
+                (parameterShape && parameterShape->isParametricShape())) {
 
             haveConvertibleShapes = true;
             break;
@@ -1154,30 +1126,32 @@ void KoPathTool::deactivate()
     m_canvasConnections.clear();
     m_pointSelection.clear();
     m_pointSelection.setSelectedShapes(QList<KoPathShape*>());
-    delete m_activeHandle;
-    m_activeHandle = 0;
-    delete m_activeSegment;
-    m_activeSegment = 0;
-    delete m_currentStrategy;
-    m_currentStrategy = 0;
+    m_activeHandle.reset();
+    m_activeSegment.reset();
+    m_currentStrategy.reset();
     d->canvas->snapGuide()->reset();
 
+    disconnect(m_actionCurvePoint, 0, this, 0);
+    disconnect(m_actionLinePoint, 0, this, 0);
+    disconnect(m_actionLineSegment, 0, this, 0);
+    disconnect(m_actionCurveSegment, 0, this, 0);
+    disconnect(m_actionAddPoint, 0, this, 0);
+    disconnect(m_actionRemovePoint, 0, this, 0);
+    disconnect(m_actionBreakPoint, 0, this, 0);
+    disconnect(m_actionBreakSegment, 0, this, 0);
+    disconnect(m_actionJoinSegment, 0, this, 0);
+    disconnect(m_actionMergePoints, 0, this, 0);
+    disconnect(m_actionConvertToPath, 0, this, 0);
+    disconnect(m_points, 0, this, 0);
+    disconnect(&m_pointSelection, 0, this, 0);
+
     KoToolBase::deactivate();
 }
 
 void KoPathTool::documentResourceChanged(int key, const QVariant & res)
 {
     if (key == KoDocumentResourceManager::HandleRadius) {
-        int oldHandleRadius = m_handleRadius;
-
-        m_handleRadius = res.toUInt();
-
-        // repaint with the bigger of old and new handle radius
-        int maxRadius = qMax(m_handleRadius, oldHandleRadius);
-        Q_FOREACH (KoPathShape *shape, m_pointSelection.selectedShapes()) {
-            QRectF controlPointRect = shape->absoluteTransformation(0).map(shape->outline()).controlPointRect();
-            repaint(controlPointRect.adjusted(-maxRadius, -maxRadius, maxRadius, maxRadius));
-        }
+        repaintDecorations();
     }
 }
 
@@ -1189,15 +1163,6 @@ void KoPathTool::pointSelectionChanged()
     emit selectionChanged(m_pointSelection.hasSelection());
 }
 
-void KoPathTool::repaint(const QRectF &repaintRect)
-{
-    Q_D(KoToolBase);
-    //debugFlake <<"KoPathTool::repaint(" << repaintRect <<")" << m_handleRadius;
-    // widen border to take antialiasing into account
-    qreal radius = m_handleRadius + 1;
-    d->canvas->updateCanvas(repaintRect.adjusted(-radius, -radius, radius, radius));
-}
-
 namespace {
 void addActionsGroupIfEnabled(QMenu *menu, QAction *a1, QAction *a2)
 {
