@@ -51,6 +51,7 @@ KisGeneratorRegistry* KisGeneratorRegistry::instance()
 {
     KisGeneratorRegistry *reg = qApp->findChild<KisGeneratorRegistry *>(QString());
     if (!reg) {
+        dbgRegistry << "initializing KisGeneratorRegistry";
         reg = new KisGeneratorRegistry(qApp);
         KoPluginLoader::instance()->load("Krita/Generator", "Type == 'Service' and ([X-Krita-Version] == 28)");
     }
@@ -73,7 +74,7 @@ void KisGeneratorRegistry::add(const QString &id, KisGeneratorSP item)
 KisFilterConfigurationSP KisGeneratorRegistry::cloneConfiguration(const KisFilterConfigurationSP kfc)
 {
     KisGeneratorSP filter = value(kfc->name());
-    KisFilterConfigurationSP newkfc(filter->defaultConfiguration());
+    KisFilterConfigurationSP newkfc(filter->factoryConfiguration());
     newkfc->fromXML(kfc->toXML());
     return newkfc;
 }
