@@ -57,12 +57,12 @@ public:
     }
 
     QRect exactBounds() {
-        return m_exactBoundsCache.getValue();
+        return m_exactBoundsCache.getValue(m_paintDevice->defaultBounds()->wrapAroundMode());
     }
 
     QRect exactBoundsAmortized() {
         QRect bounds;
-        bool result = m_exactBoundsCache.tryGetValue(bounds);
+        bool result = m_exactBoundsCache.tryGetValue(bounds, m_paintDevice->defaultBounds()->wrapAroundMode());
 
         if (!result) {
             /**
@@ -78,17 +78,21 @@ public:
     }
 
     QRect nonDefaultPixelArea() {
-        return m_nonDefaultPixelAreaCache.getValue();
+        return m_nonDefaultPixelAreaCache.getValue(m_paintDevice->defaultBounds()->wrapAroundMode());
     }
 
-    QRegion region() {
-        return m_regionCache.getValue();
+    KisRegion region() {
+        return m_regionCache.getValue(m_paintDevice->defaultBounds()->wrapAroundMode());
     }
 
     QImage createThumbnail(qint32 w, qint32 h, qreal oversample, KoColorConversionTransformation::Intent renderingIntent, KoColorConversionTransformation::ConversionFlags conversionFlags) {
         QImage thumbnail;
 
-        if(m_thumbnailsValid) {
+        if (h == 0 || w == 0) {
+            return thumbnail;
+        }
+
+        if (m_thumbnailsValid) {
             thumbnail = findThumbnail(w, h, oversample);
         }
         else {
@@ -96,7 +100,7 @@ public:
             m_thumbnailsValid = true;
         }
 
-        if(thumbnail.isNull()) {
+        if (thumbnail.isNull()) {
             thumbnail = m_paintDevice->createThumbnail(w, h, QRect(), oversample, renderingIntent, conversionFlags);
             cacheThumbnail(w, h, oversample, thumbnail);
         }
@@ -124,30 +128,30 @@ private:
 private:
     KisPaintDevice *m_paintDevice;
 
-    struct ExactBoundsCache : KisLockFreeCache<QRect> {
+    struct ExactBoundsCache : KisLockFreeCacheWithModeConsistency<QRect, bool> {
         ExactBoundsCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRect calculateNewValue() const {
+        QRect calculateNewValue() const override {
             return m_paintDevice->calculateExactBounds(false);
         }
     private:
         KisPaintDevice *m_paintDevice;
     };
 
-    struct NonDefaultPixelCache : KisLockFreeCache<QRect> {
+    struct NonDefaultPixelCache : KisLockFreeCacheWithModeConsistency<QRect, bool> {
         NonDefaultPixelCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRect calculateNewValue() const {
+        QRect calculateNewValue() const override {
             return m_paintDevice->calculateExactBounds(true);
         }
     private:
         KisPaintDevice *m_paintDevice;
     };
 
-    struct RegionCache : KisLockFreeCache<QRegion> {
+    struct RegionCache : KisLockFreeCacheWithModeConsistency<KisRegion, bool> {
         RegionCache(KisPaintDevice *paintDevice) : m_paintDevice(paintDevice) {}
 
-        QRegion calculateNewValue() const {
+        KisRegion calculateNewValue() const override {
             return m_paintDevice->dataManager()->region();
         }
     private:
