@@ -23,14 +23,15 @@
 #include "kis_base_node.h"
 
 #include "kis_types.h"
-#include "kis_mask.h"
+#include "kis_effect_mask.h"
+#include "KisDecoratedNodeInterface.h"
 
 /**
  * An selection mask is a single channel mask that applies a
  * particular selection to the layer the mask belongs to. A selection
  * can contain both vector and pixel selection components.
 */
-class KRITAIMAGE_EXPORT KisSelectionMask : public KisMask
+class KRITAIMAGE_EXPORT KisSelectionMask : public KisEffectMask, public KisDecoratedNodeInterface
 {
     Q_OBJECT
 public:
@@ -39,7 +40,7 @@ public:
      * Create an empty selection mask. There is filter and no layer
      * associated with this mask.
      */
-    KisSelectionMask(KisImageWSP image);
+    KisSelectionMask(KisImageWSP image, const QString &name = QString());
 
     ~KisSelectionMask() override;
     KisSelectionMask(const KisSelectionMask& rhs);
@@ -63,6 +64,12 @@ public:
     bool active() const;
     void setActive(bool active);
 
+    QRect needRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const override;
+    QRect changeRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const override;
+
+    QRect extent() const override;
+    QRect exactBounds() const override;
+
     /**
      * This method works like the one in KisSelection, but it
      * compressed the incoming events instead of processing each of
@@ -70,10 +77,27 @@ public:
      */
     void notifySelectionChangedCompressed();
 
-private:
-    Q_PRIVATE_SLOT(m_d, void slotSelectionChangedCompressed());
+    bool decorationsVisible() const override;
+    void setDecorationsVisible(bool value, bool update) override;
+    using KisDecoratedNodeInterface::setDecorationsVisible;
 
-    KisImageWSP image() const;
+    void setDirty(const QVector<QRect> &rects) override;
+    using KisEffectMask::setDirty;
+
+protected:
+    void flattenSelectionProjection(KisSelectionSP selection, const QRect &dirtyRect) const override;
+
+    void mergeInMaskInternal(KisPaintDeviceSP projection,
+                             KisSelectionSP effectiveSelection,
+                             const QRect &applyRect, const QRect &preparedNeedRect,
+                             KisNode::PositionToFilthy maskPos) const override;
+
+    bool paintsOutsideSelection() const override;
+
+
+private:
+    Q_PRIVATE_SLOT(m_d, void slotSelectionChangedCompressed())
+    Q_PRIVATE_SLOT(m_d, void slotConfigChanged())
 
     struct Private;
     Private * const m_d;
