@@ -21,6 +21,7 @@
 #include <QDebug>
 #include "kis_keyframe_channel.h"
 #include "kis_node.h"
+#include "kis_layer_utils.h"
 
 struct KisTimeRangeStaticRegistrar {
     KisTimeRangeStaticRegistrar() {
@@ -37,23 +38,32 @@ QDebug operator<<(QDebug dbg, const KisTimeRange &r)
     return dbg.space();
 }
 
-void KisTimeRange::calculateTimeRangeRecursive(const KisNode *node, int time, KisTimeRange &range, bool exclusive)
+KisTimeRange KisTimeRange::calculateIdenticalFramesRecursive(const KisNode *node, int time)
 {
-    if (!node->visible()) return;
+    KisTimeRange range = KisTimeRange::infinite(0);
 
-    if (exclusive) {
-        // Intersection
-        range &= calculateNodeIdenticalFrames(node, time);
-    } else {
-        // Union
-        range |= calculateNodeAffectedFrames(node, time);
-    }
+    KisLayerUtils::recursiveApplyNodes(node,
+        [&range, time] (const KisNode *node) {
+            if (node->visible()) {
+                range &= calculateNodeIdenticalFrames(node, time);
+            }
+    });
 
-    KisNodeSP child = node->firstChild();
-    while (child) {
-        calculateTimeRangeRecursive(child, time, range, exclusive);
-        child = child->nextSibling();
-    }
+    return range;
+}
+
+KisTimeRange KisTimeRange::calculateAffectedFramesRecursive(const KisNode *node, int time)
+{
+    KisTimeRange range;
+
+    KisLayerUtils::recursiveApplyNodes(node,
+        [&range, time] (const KisNode *node) {
+            if (node->visible()) {
+                range |= calculateNodeIdenticalFrames(node, time);
+            }
+    });
+
+    return range;
 }
 
 KisTimeRange KisTimeRange::calculateNodeIdenticalFrames(const KisNode *node, int time)
@@ -63,12 +73,6 @@ KisTimeRange KisTimeRange::calculateNodeIdenticalFrames(const KisNode *node, int
     const QMap<QString, KisKeyframeChannel*> channels =
         node->keyframeChannels();
 
-    if (channels.isEmpty() ||
-        !channels.contains(KisKeyframeChannel::Content.id())) {
-
-        return range;
-    }
-
     Q_FOREACH (const KisKeyframeChannel *channel, channels) {
         // Intersection
         range &= channel->identicalFrames(time);
@@ -86,9 +90,11 @@ KisTimeRange KisTimeRange::calculateNodeAffectedFrames(const KisNode *node, int
     const QMap<QString, KisKeyframeChannel*> channels =
         node->keyframeChannels();
 
+    // TODO: channels should report to the image which channel exactly has changed
+    //       to avoid the dirty range to be stretched into infinity!
+
     if (channels.isEmpty() ||
         !channels.contains(KisKeyframeChannel::Content.id())) {
-
         range = KisTimeRange::infinite(0);
         return range;
     }
@@ -132,7 +138,7 @@ bool loadValue(const QDomElement &parent, const QString &tag, KisTimeRange *rang
     int end = toInt(e.attribute("to", "-1"));
 
     if (start == -1) {
-        range = new KisTimeRange();
+        *range = KisTimeRange();
     } else if (end == -1) {
         *range = KisTimeRange::infinite(start);
     } else {
