@@ -58,7 +58,7 @@ class KRITAIMAGE_EXPORT KisAnimatedTransformParamsInterface
 public:
     virtual ~KisAnimatedTransformParamsInterface();
 
-    virtual KisKeyframeChannel *getKeyframeChannel(const QString &id, KisDefaultBoundsBaseSP defaultBounds) = 0;
+    virtual KisKeyframeChannel *getKeyframeChannel(const QString &id, KisNodeSP parent) = 0;
 };
 
 class QDomElement;
@@ -69,31 +69,31 @@ public:
     KisDumbTransformMaskParams();
     KisDumbTransformMaskParams(const QTransform &transform);
     KisDumbTransformMaskParams(bool isHidden);
-    ~KisDumbTransformMaskParams();
+    ~KisDumbTransformMaskParams() override;
 
 
-    QTransform finalAffineTransform() const;
-    bool isAffine() const;
-    bool isHidden() const;
-    void transformDevice(KisNodeSP node, KisPaintDeviceSP src, KisPaintDeviceSP dst) const;
+    QTransform finalAffineTransform() const override;
+    bool isAffine() const override;
+    bool isHidden() const override;
+    void transformDevice(KisNodeSP node, KisPaintDeviceSP src, KisPaintDeviceSP dst) const override;
 
-    QString id() const;
-    void toXML(QDomElement *e) const;
+    QString id() const override;
+    void toXML(QDomElement *e) const override;
     static KisTransformMaskParamsInterfaceSP fromXML(const QDomElement &e);
 
-    void translate(const QPointF &offset);
+    void translate(const QPointF &offset) override;
 
     // for tesing purposes only
     QTransform testingGetTransform() const;
     void testingSetTransform(const QTransform &t);
 
-    QRect nonAffineChangeRect(const QRect &rc);
-    QRect nonAffineNeedRect(const QRect &rc, const QRect &srcBounds);
+    QRect nonAffineChangeRect(const QRect &rc) override;
+    QRect nonAffineNeedRect(const QRect &rc, const QRect &srcBounds) override;
 
     bool isAnimated() const;
     KisKeyframeChannel *getKeyframeChannel(const QString &id, KisDefaultBoundsBaseSP defaultBounds);
-    void clearChangedFlag();
-    bool hasChanged() const;
+    void clearChangedFlag() override;
+    bool hasChanged() const override;
 
 private:
     struct Private;
