@@ -43,22 +43,29 @@
 
 #include "kis_ls_utils.h"
 #include "kis_multiple_projection.h"
+#include "kis_cached_paint_device.h"
+#include "krita_utils.h"
+#include "KisLayerStyleKnockoutBlower.h"
+
 
 namespace {
 
 int borderSize(psd_stroke_position position, int size)
 {
-    int border = 0;
+    int border = 1;
+
+    // The need rect border should not depend on the postition;
+    // instead it should extend the area all the time. Otherwise
+    // small changes on the borderline will not be propagated to
+    // the inner side of the blob. See bug 429165.
 
     switch (position) {
     case psd_stroke_outside:
-        border = 2 * size + 1;
-        break;
-    case psd_stroke_center:
+    case psd_stroke_inside:
         border = size + 1;
         break;
-    case psd_stroke_inside:
-        border = 1;
+    case psd_stroke_center:
+        border = qCeil(0.5 * size) + 1;
         break;
     }
 
@@ -73,8 +80,19 @@ KisLsStrokeFilter::KisLsStrokeFilter()
 {
 }
 
+KisLsStrokeFilter::KisLsStrokeFilter(const KisLsStrokeFilter &rhs)
+    : KisLayerStyleFilter(rhs)
+{
+}
+
+KisLayerStyleFilter *KisLsStrokeFilter::clone() const
+{
+    return new KisLsStrokeFilter(*this);
+}
+
 void KisLsStrokeFilter::applyStroke(KisPaintDeviceSP srcDevice,
                                     KisMultipleProjection *dst,
+                                    KisLayerStyleKnockoutBlower *blower,
                                     const QRect &applyRect,
                                     const psd_layer_effects_stroke *config,
                                     KisLayerStyleFilterEnvironment *env) const
@@ -83,48 +101,53 @@ void KisLsStrokeFilter::applyStroke(KisPaintDeviceSP srcDevice,
 
     const QRect needRect = kisGrowRect(applyRect, borderSize(config->position(), config->size()));
 
-    KisSelectionSP baseSelection = KisLsUtils::selectionFromAlphaChannel(srcDevice, needRect);
+    KisSelectionSP baseSelection = blower->knockoutSelectionLazy();
     KisPixelSelectionSP selection = baseSelection->pixelSelection();
 
+    KisCachedSelection::Guard s1(*env->cachedSelection());
+    KisPixelSelectionSP dilatedSelection = s1.selection()->pixelSelection();
+    KisLsUtils::selectionFromAlphaChannel(srcDevice, s1.selection(), needRect);
+
     {
-        KisPixelSelectionSP knockOutSelection = new KisPixelSelection(new KisSelectionEmptyBounds(0));
-        knockOutSelection->makeCloneFromRough(selection, needRect);
+        KisCachedSelection::Guard s2(*env->cachedSelection());
+        KisPixelSelectionSP erodedSelection = s2.selection()->pixelSelection();
+        erodedSelection->makeCloneFromRough(dilatedSelection, needRect);
 
         if (config->position() == psd_stroke_outside) {
-            KisGaussianKernel::applyDilate(selection, needRect, 2 * config->size(), QBitArray(), 0, true);
+            KisGaussianKernel::applyDilate(dilatedSelection, needRect, config->size(), QBitArray(), 0, true);
         } else if (config->position() == psd_stroke_inside) {
-            KisGaussianKernel::applyErodeU8(knockOutSelection, needRect, 2 * config->size(), QBitArray(), 0, true);
+            KisGaussianKernel::applyErodeU8(erodedSelection, needRect, config->size(), QBitArray(), 0, true);
         } else if (config->position() == psd_stroke_center) {
-            KisGaussianKernel::applyDilate(selection, needRect, config->size(), QBitArray(), 0, true);
-            KisGaussianKernel::applyErodeU8(knockOutSelection, needRect, config->size(), QBitArray(), 0, true);
+            KisGaussianKernel::applyDilate(dilatedSelection, needRect, 0.5 * config->size(), QBitArray(), 0, true);
+            KisGaussianKernel::applyErodeU8(erodedSelection, needRect, 0.5 * config->size(), QBitArray(), 0, true);
         }
 
         KisPainter gc(selection);
+
+        gc.setCompositeOp(COMPOSITE_COPY);
+        gc.bitBlt(applyRect.topLeft(), dilatedSelection, applyRect);
+
         gc.setCompositeOp(COMPOSITE_ERASE);
-        gc.bitBlt(needRect.topLeft(), knockOutSelection, needRect);
+        gc.bitBlt(applyRect.topLeft(), erodedSelection, applyRect);
         gc.end();
     }
 
-    KisPaintDeviceSP fillDevice = new KisPaintDevice(srcDevice->colorSpace());
-    KisLsUtils::fillOverlayDevice(fillDevice, applyRect, config, env);
-
-
     const QString compositeOp = config->blendMode();
-    const quint8 opacityU8 = 255.0 / 100.0 * config->opacity();
+    const quint8 opacityU8 = quint8(qRound(255.0 / 100.0 * config->opacity()));
     KisPaintDeviceSP dstDevice = dst->getProjection(KisMultipleProjection::defaultProjectionId(),
                                                     compositeOp,
                                                     opacityU8,
                                                     QBitArray(),
                                                     srcDevice);
-
-    KisPainter::copyAreaOptimized(applyRect.topLeft(), fillDevice, dstDevice, applyRect, baseSelection);
+    KisLsUtils::fillOverlayDevice(dstDevice, applyRect, config, env);
 }
 
 void KisLsStrokeFilter::processDirectly(KisPaintDeviceSP src,
                                         KisMultipleProjection *dst,
-                                         const QRect &applyRect,
-                                         KisPSDLayerStyleSP style,
-                                         KisLayerStyleFilterEnvironment *env) const
+                                        KisLayerStyleKnockoutBlower *blower,
+                                        const QRect &applyRect,
+                                        KisPSDLayerStyleSP style,
+                                        KisLayerStyleFilterEnvironment *env) const
 {
     Q_UNUSED(env);
     KIS_ASSERT_RECOVER_RETURN(style);
@@ -133,7 +156,7 @@ void KisLsStrokeFilter::processDirectly(KisPaintDeviceSP src,
     if (!KisLsUtils::checkEffectEnabled(config, dst)) return;
 
     KisLsUtils::LodWrapper<psd_layer_effects_stroke> w(env->currentLevelOfDetail(), config);
-    applyStroke(src, dst, applyRect, w.config, env);
+    applyStroke(src, dst, blower, applyRect, w.config, env);
 }
 
 QRect KisLsStrokeFilter::neededRect(const QRect &rect, KisPSDLayerStyleSP style, KisLayerStyleFilterEnvironment *env) const
