@@ -17,6 +17,7 @@
  */
 #include <QUrl>
 #include <QScopedPointer>
+#include <QUuid>
 
 #include <KoColorSpace.h>
 #include <KoColorSpaceRegistry.h>
@@ -25,8 +26,6 @@
 #include <KisDocument.h>
 #include <KisMimeDatabase.h>
 #include <KisPart.h>
-#include <kis_change_profile_visitor.h>
-#include <kis_colorspace_convert_visitor.h>
 #include <kis_image.h>
 #include <kis_types.h>
 #include <kis_node.h>
@@ -47,6 +46,9 @@
 #include <kis_meta_data_merge_strategy.h>
 #include <kis_meta_data_merge_strategy_registry.h>
 #include <kis_filter_strategy.h>
+#include <commands/kis_node_compositeop_command.h>
+#include <commands/kis_image_layer_add_command.h>
+#include <kis_processing_applicator.h>
 
 #include <kis_raster_keyframe_channel.h>
 #include <kis_keyframe.h>
@@ -84,6 +86,40 @@ Node::Node(KisImageSP image, KisNodeSP node, QObject *parent)
     d->node = node;
 }
 
+Node *Node::createNode(KisImageSP image, KisNodeSP node, QObject *parent)
+{
+    if (node.isNull()) {
+        return 0;
+    }
+    if (node->inherits("KisGroupLayer")) {
+        return new GroupLayer(dynamic_cast<KisGroupLayer*>(node.data()));
+    }
+    else if (node->inherits("KisCloneLayer")) {
+        return new CloneLayer(dynamic_cast<KisCloneLayer*>(node.data()));
+    }
+    else if (node->inherits("KisFileLayer")) {
+        return new FileLayer(dynamic_cast<KisFileLayer*>(node.data()));
+    }
+    else if (node->inherits("KisAdjustmentLayer")) {
+        return new FilterLayer(dynamic_cast<KisAdjustmentLayer*>(node.data()));
+    }
+    else if (node->inherits("KisGeneratorLayer")) {
+        return new FillLayer(dynamic_cast<KisGeneratorLayer*>(node.data()));
+    }
+    else if (node->inherits("KisShapeLayer")) {
+        return new VectorLayer(dynamic_cast<KisShapeLayer*>(node.data()));
+    }
+    else if (node->inherits("KisFilterMask")) {
+        return new FilterMask(image, dynamic_cast<KisFilterMask*>(node.data()));
+    }
+    else if (node->inherits("KisSelectionMask")) {
+        return new SelectionMask(image, dynamic_cast<KisSelectionMask*>(node.data()));
+    }
+    else {
+        return new Node(image, node, parent);
+    }
+}
+
 Node::~Node()
 {
     delete d;
@@ -103,7 +139,7 @@ bool Node::operator!=(const Node &other) const
 Node *Node::clone() const
 {
     KisNodeSP clone = d->node->clone();
-    Node *node = new Node(0, clone);
+    Node *node = Node::createNode(0, clone);
     return node;
 }
 
@@ -138,7 +174,13 @@ QString Node::blendingMode() const
 void Node::setBlendingMode(QString value)
 {
     if (!d->node) return;
-    d->node->setCompositeOpId(value);
+
+    KUndo2Command *cmd = new KisNodeCompositeOpCommand(d->node,
+                                                       d->node->compositeOpId(),
+                                                       value);
+
+    KisProcessingApplicator::runSingleCommandStroke(d->image, cmd);
+    d->image->waitForDone();
 }
 
 
@@ -174,12 +216,19 @@ QList<Node*> Node::childNodes() const
 bool Node::addChildNode(Node *child, Node *above)
 {
     if (!d->node) return false;
+
+    KUndo2Command *cmd = 0;
+
     if (above) {
-        return d->image->addNode(child->node(), d->node, above->node());
-    }
-    else {
-        return d->image->addNode(child->node(), d->node, d->node->childCount());
+        cmd = new KisImageLayerAddCommand(d->image, child->node(), d->node, above->node());
+    } else {
+        cmd = new KisImageLayerAddCommand(d->image, child->node(), d->node, d->node->childCount());
     }
+
+    KisProcessingApplicator::runSingleCommandStroke(d->image, cmd);
+    d->image->waitForDone();
+
+    return true;
 }
 
 bool Node::removeChildNode(Node *child)
@@ -241,26 +290,22 @@ bool Node::setColorProfile(const QString &colorProfile)
     if (!d->node->inherits("KisLayer")) return false;
     KisLayer *layer = qobject_cast<KisLayer*>(d->node.data());
     const KoColorProfile *profile = KoColorSpaceRegistry::instance()->profileByName(colorProfile);
-    const KoColorSpace *srcCS = layer->colorSpace();
-    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(srcCS->colorModelId().id(),
-                                                                             srcCS->colorDepthId().id(),
-                                                                             profile);
-    KisChangeProfileVisitor v(srcCS, dstCs);
-    return layer->accept(v);
+    bool result = d->image->assignLayerProfile(layer, profile);
+    d->image->waitForDone();
+    return result;
 }
 
 bool Node::setColorSpace(const QString &colorModel, const QString &colorDepth, const QString &colorProfile)
 {
     if (!d->node) return false;
     if (!d->node->inherits("KisLayer")) return false;
-    KisLayer *layer = qobject_cast<KisLayer*>(d->node.data());
     const KoColorProfile *profile = KoColorSpaceRegistry::instance()->profileByName(colorProfile);
-    const KoColorSpace *srcCS = layer->colorSpace();
     const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorModel,
                                                                              colorDepth,
                                                                              profile);
-    KisColorSpaceConvertVisitor v(d->image, srcCS, dstCs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
-    return layer->accept(v);
+    d->image->convertLayerColorSpace(d->node, dstCs, KoColorConversionTransformation::internalRenderingIntent(), KoColorConversionTransformation::internalConversionFlags());
+    d->image->waitForDone();
+    return true;
 }
 
 bool Node::animated() const
@@ -361,7 +406,8 @@ void Node::setOpacity(int value)
 Node* Node::parentNode() const
 {
     if (!d->node) return 0;
-    return new Node(d->image, d->node->parent());
+    if (!d->node->parent()) return 0;
+    return Node::createNode(d->image, d->node->parent());
 }
 
 QString Node::type() const
@@ -474,7 +520,7 @@ QByteArray Node::pixelDataAtTime(int x, int y, int w, int h, int time) const
     if (!rkc) return ba;
     KisKeyframeSP frame = rkc->keyframeAt(time);
     if (!frame) return ba;
-    KisPaintDeviceSP dev = d->node->paintDevice();
+    KisPaintDeviceSP dev = new KisPaintDevice(*d->node->paintDevice(), KritaUtils::DeviceCopyMode::CopySnapshot);
     if (!dev) return ba;
 
     rkc->fetchFrame(frame, dev);
@@ -536,7 +582,7 @@ bool Node::remove()
 Node* Node::duplicate()
 {
     if (!d->node) return 0;
-    return new Node(d->image, d->node->clone());
+    return Node::createNode(d->image, d->node->clone());
 }
 
 bool Node::save(const QString &filename, double xRes, double yRes, const InfoObject &exportConfiguration, const QRect &exportRect)
@@ -580,7 +626,7 @@ Node* Node::mergeDown()
     d->image->mergeDown(qobject_cast<KisLayer*>(d->node.data()), KisMetaData::MergeStrategyRegistry::instance()->get("Drop"));
     d->image->waitForDone();
 
-    return new Node(d->image, d->node->prevSibling());
+    return Node::createNode(d->image, d->node->prevSibling());
 }
 
 void Node::scaleNode(QPointF origin, int width, int height, QString strategy)
@@ -599,6 +645,7 @@ void Node::scaleNode(QPointF origin, int width, int height, QString strategy)
                         qreal(width) / bounds.width(),
                         qreal(height) / bounds.height(),
                         actualStrategy, 0);
+    d->image->waitForDone();
 }
 
 void Node::rotateNode(double radians)
@@ -608,6 +655,7 @@ void Node::rotateNode(double radians)
     if (!d->node->parent()) return;
 
     d->image->rotateNode(d->node, radians, 0);
+    d->image->waitForDone();
 }
 
 void Node::cropNode(int x, int y, int w, int h)
@@ -618,6 +666,7 @@ void Node::cropNode(int x, int y, int w, int h)
 
     QRect rect = QRect(x, y, w, h);
     d->image->cropNode(d->node, rect);
+    d->image->waitForDone();
 }
 
 void Node::shearNode(double angleX, double angleY)
@@ -627,6 +676,7 @@ void Node::shearNode(double angleX, double angleY)
     if (!d->node->parent()) return;
 
     d->image->shearNode(d->node, angleX, angleY, 0);
+    d->image->waitForDone();
 }
 
 QImage Node::thumbnail(int w, int h)
@@ -635,6 +685,12 @@ QImage Node::thumbnail(int w, int h)
     return d->node->createThumbnail(w, h);
 }
 
+QUuid Node::uniqueId() const
+{
+    if (!d->node) return QUuid();
+    return d->node->uuid();
+}
+
 KisPaintDeviceSP Node::paintDevice() const
 {
     return d->node->paintDevice();
