@@ -34,8 +34,11 @@
 #include "KoColorSpaceRegistry.h"
 #include "KoMixColorsOp.h"
 
+#include "kis_dom_utils.h"
+
 #include <math.h>
 #include <KoColorModelStandardIds.h>
+#include <KoXmlNS.h>
 
 KoStopGradient::KoStopGradient(const QString& filename)
     : KoAbstractGradient(filename)
@@ -46,7 +49,7 @@ KoStopGradient::~KoStopGradient()
 {
 }
 
-bool KoStopGradient::operator==(const KoStopGradient &rhs) const
+bool KoStopGradient::operator==(const KoStopGradient& rhs) const
 {
     return
         *colorSpace() == *rhs.colorSpace() &&
@@ -75,7 +78,7 @@ bool KoStopGradient::load()
     return res;
 }
 
-bool KoStopGradient::loadFromDevice(QIODevice *dev)
+bool KoStopGradient::loadFromDevice(QIODevice* dev)
 {
     QString strExt;
     const int result = filename().lastIndexOf('.');
@@ -85,12 +88,7 @@ bool KoStopGradient::loadFromDevice(QIODevice *dev)
     QByteArray ba = dev->readAll();
 
     QBuffer buf(&ba);
-    if (strExt == ".kgr") {
-        loadKarbonGradient(&buf);
-    }
-    else if (strExt == ".svg") {
-        loadSvgGradient(&buf);
-    }
+    loadSvgGradient(&buf);
     if (m_stops.count() >= 2) {
         setValid(true);
     }
@@ -101,7 +99,7 @@ bool KoStopGradient::loadFromDevice(QIODevice *dev)
 bool KoStopGradient::save()
 {
     QFile fileOut(filename());
-    if (! fileOut.open(QIODevice::WriteOnly))
+    if (!fileOut.open(QIODevice::WriteOnly))
         return false;
 
     bool retval = saveToDevice(&fileOut);
@@ -137,8 +135,8 @@ QGradient* KoStopGradient::toQGradient() const
     }
     QColor color;
     for (QList<KoGradientStop>::const_iterator i = m_stops.begin(); i != m_stops.end(); ++i) {
-        i->second.toQColor(&color);
-        gradient->setColorAt(i->first , color);
+        i->color.toQColor(&color);
+        gradient->setColorAt(i->position, color);
     }
 
     gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
@@ -147,107 +145,104 @@ QGradient* KoStopGradient::toQGradient() const
     return gradient;
 }
 
+bool KoStopGradient::stopsAt(KoGradientStop& leftStop, KoGradientStop& rightStop, qreal t) const
+{
+    if (!m_stops.count())
+        return false;
+    if (t <= m_stops.first().position || m_stops.count() == 1) {
+        // we have only one stop or t is before the first stop
+        leftStop = m_stops.first();
+        rightStop = KoGradientStop(-std::numeric_limits<double>::infinity(), leftStop.color, leftStop.type);
+        return true;
+    } else if (t >= m_stops.last().position) {
+        // t is after the last stop
+        rightStop = m_stops.last();
+        leftStop = KoGradientStop(std::numeric_limits<double>::infinity(), rightStop.color, rightStop.type);
+        return true;
+    } else {
+        // we have at least two color stops
+        // -> find the two stops which frame our t
+        auto it = std::lower_bound(m_stops.begin(), m_stops.end(), KoGradientStop(t, KoColor(), COLORSTOP), [](const KoGradientStop& a, const KoGradientStop& b) {
+            return a.position < b.position;
+            });
+        leftStop = *(it - 1);
+        rightStop = *(it);
+        return true;
+    }
+}
+
 void KoStopGradient::colorAt(KoColor& dst, qreal t) const
 {
     KoColor buffer;
 
-    if (! m_stops.count())
-        return;
-    if (t <= m_stops.first().first || m_stops.count() == 1) {
-        // we have only one stop or t is before the first stop
-        // -> use the color of the first stop
-        dst.fromKoColor(m_stops.first().second);
-    } else if (t >= m_stops.last().first) {
-        // t is after the last stop
-        // -> use the color of the last stop
-        dst.fromKoColor(m_stops.last().second);
+    KoGradientStop leftStop, rightStop;
+    if (!stopsAt(leftStop, rightStop, t)) return;
+
+    const KoColorSpace* mixSpace = KoColorSpaceRegistry::instance()->rgb8(dst.colorSpace()->profile());
+
+    KoColor startDummy, endDummy;
+    if (mixSpace) {
+        startDummy = KoColor(leftStop.color, mixSpace);
+        endDummy = KoColor(rightStop.color, mixSpace);
     } else {
-        // we have at least two color stops
-        // -> find the two stops which frame our t
-        QList<KoGradientStop>::const_iterator stop = m_stops.begin();
-        QList<KoGradientStop>::const_iterator lastStop = m_stops.end();
-        // we already checked the first stop, so we start at the second
-        for (++stop; stop != lastStop; ++stop) {
-            // we break at the stop which is just after our t
-            if (stop->first > t)
-                break;
-        }
-
-        //if ( *buffer.colorSpace() != *colorSpace()) {
-        //    buffer = KoColor(colorSpace());
-        //}
-        //hack to get a color space with the bitdepth of the gradients(8bit), but with the colour profile of the image//
-        const KoColorSpace* mixSpace = KoColorSpaceRegistry::instance()->rgb8(dst.colorSpace()->profile());
-
-        const KoGradientStop& leftStop = *(stop - 1);
-        const KoGradientStop& rightStop = *(stop);
-
-        KoColor startDummy, endDummy;
-        if (mixSpace){
-            startDummy = KoColor(leftStop.second, mixSpace);
-            endDummy = KoColor(rightStop.second, mixSpace);
-        } else {
-            startDummy = leftStop.second;
-            endDummy = rightStop.second;
-        }
-        const quint8 *colors[2];
-        colors[0] = startDummy.data();
-        colors[1] = endDummy.data();
-
-        qreal localT;
-        qreal stopDistance = rightStop.first - leftStop.first;
-        if (stopDistance < DBL_EPSILON) {
-            localT = 0.5;
-        } else {
-            localT = (t - leftStop.first) / stopDistance;
-        }
-        qint16 colorWeights[2];
-        colorWeights[0] = static_cast<quint8>((1.0 - localT) * 255 + 0.5);
-        colorWeights[1] = 255 - colorWeights[0];
-
-
-        //check if our mixspace exists, it doesn't at startup.
-        if (mixSpace){
-            if (*buffer.colorSpace() != *mixSpace) {
-                buffer = KoColor(mixSpace);
-            }
-            mixSpace->mixColorsOp()->mixColors(colors, colorWeights, 2, buffer.data());
-        }
-        else {
-            buffer = KoColor(colorSpace());
-            colorSpace()->mixColorsOp()->mixColors(colors, colorWeights, 2, buffer.data());
-        }
-
-        dst.fromKoColor(buffer);
+        startDummy = leftStop.color;
+        endDummy = rightStop.color;
     }
+    const quint8* colors[2];
+    colors[0] = startDummy.data();
+    colors[1] = endDummy.data();
+
+    qreal localT;
+    qreal stopDistance = rightStop.position - leftStop.position;
+    if (stopDistance < DBL_EPSILON) {
+        localT = 0.5;
+    } else {
+        localT = (t - leftStop.position) / stopDistance;
+    }
+    qint16 colorWeights[2];
+    colorWeights[0] = static_cast<quint8>((1.0 - localT) * 255 + 0.5);
+    colorWeights[1] = 255 - colorWeights[0];
+
+    //check if our mixspace exists, it doesn't at startup.
+    if (mixSpace) {
+        if (*buffer.colorSpace() != *mixSpace) {
+            buffer = KoColor(mixSpace);
+        }
+        mixSpace->mixColorsOp()->mixColors(colors, colorWeights, 2, buffer.data());
+    } else {
+        buffer = KoColor(colorSpace());
+        colorSpace()->mixColorsOp()->mixColors(colors, colorWeights, 2, buffer.data());
+    }
+
+    dst.fromKoColor(buffer);
 }
 
-KoStopGradient * KoStopGradient::fromQGradient(const QGradient * gradient)
+KoStopGradient* KoStopGradient::fromQGradient(const QGradient* gradient)
 {
-    if (! gradient)
+    if (!gradient)
         return 0;
 
-    KoStopGradient * newGradient = new KoStopGradient(QString());
+    KoStopGradient* newGradient = new KoStopGradient(QString());
     newGradient->setType(gradient->type());
     newGradient->setSpread(gradient->spread());
 
     switch (gradient->type()) {
     case QGradient::LinearGradient: {
-        const QLinearGradient * g = static_cast<const QLinearGradient*>(gradient);
+        const QLinearGradient* g = static_cast<const QLinearGradient*>(gradient);
         newGradient->m_start = g->start();
         newGradient->m_stop = g->finalStop();
         newGradient->m_focalPoint = g->start();
         break;
     }
     case QGradient::RadialGradient: {
-        const QRadialGradient * g = static_cast<const QRadialGradient*>(gradient);
+        const QRadialGradient* g = static_cast<const QRadialGradient*>(gradient);
         newGradient->m_start = g->center();
         newGradient->m_stop = g->center() + QPointF(g->radius(), 0);
         newGradient->m_focalPoint = g->focalPoint();
         break;
     }
     case QGradient::ConicalGradient: {
-        const QConicalGradient * g = static_cast<const QConicalGradient*>(gradient);
+        const QConicalGradient* g = static_cast<const QConicalGradient*>(gradient);
         qreal radian = g->angle() * M_PI / 180.0;
         newGradient->m_start = g->center();
         newGradient->m_stop = QPointF(100.0 * cos(radian), 100.0 * sin(radian));
@@ -259,10 +254,10 @@ KoStopGradient * KoStopGradient::fromQGradient(const QGradient * gradient)
         return 0;
     }
 
-    Q_FOREACH (const QGradientStop & stop, gradient->stops()) {
+    Q_FOREACH(const QGradientStop & stop, gradient->stops()) {
         KoColor color(newGradient->colorSpace());
         color.fromQColor(stop.second);
-        newGradient->m_stops.append(KoGradientStop(stop.first, color));
+        newGradient->m_stops.append(KoGradientStop(stop.first, color, COLORSTOP));
     }
 
     newGradient->setValid(true);
@@ -273,11 +268,15 @@ KoStopGradient * KoStopGradient::fromQGradient(const QGradient * gradient)
 void KoStopGradient::setStops(QList< KoGradientStop > stops)
 {
     m_stops.clear();
+    m_hasVariableStops = false;
     KoColor color;
-    Q_FOREACH (const KoGradientStop & stop, stops) {
-        color = stop.second;
+    Q_FOREACH(const KoGradientStop & stop, stops) {
+        color = stop.color;
         color.convertTo(colorSpace());
-        m_stops.append(KoGradientStop(stop.first, color));
+        m_stops.append(KoGradientStop(stop.position, color, stop.type));
+        if (stop.type != COLORSTOP) {
+            m_hasVariableStops = true;
+        }
     }
     updatePreview();
 }
@@ -287,34 +286,47 @@ QList<KoGradientStop> KoStopGradient::stops() const
     return m_stops;
 }
 
-void KoStopGradient::loadKarbonGradient(QIODevice *file)
+bool KoStopGradient::hasVariableColors() const {
+    return m_hasVariableStops;
+}
+
+void KoStopGradient::setVariableColors(const KoColor& foreground, const KoColor& background) {
+    KoColor color;
+    for (int i = 0; i < m_stops.count(); i++){
+        if (m_stops[i].type == FOREGROUNDSTOP) {
+            color = foreground;
+        } else if (m_stops[i].type == BACKGROUNDSTOP) {
+            color = background;
+        } else continue;
+        color.convertTo(colorSpace());
+        m_stops[i].color = color;
+    }
+    updatePreview();
+}
+
+void KoStopGradient::bakeVariableColors(const KoColor& foreground, const KoColor& background) {
+    const KoColor fgColor = foreground.convertedTo(colorSpace());
+    const KoColor bgColor = background.convertedTo(colorSpace());
+
+    for (int i = 0; i < m_stops.count(); i++){
+        if (m_stops[i].type == FOREGROUNDSTOP) {
+            m_stops[i].color = fgColor;
+            m_stops[i].type = COLORSTOP;
+        } else if (m_stops[i].type == BACKGROUNDSTOP) {
+            m_stops[i].color = bgColor;
+            m_stops[i].type = COLORSTOP;
+        } else continue;
+    }
+    updatePreview();
+}
+
+void KoStopGradient::loadSvgGradient(QIODevice* file)
 {
     QDomDocument doc;
 
     if (!(doc.setContent(file))) {
         file->close();
-        setValid(false);
-        return;
-    }
-
-    QDomElement e;
-    QDomNode n = doc.documentElement().firstChild();
-
-    if (!n.isNull()) {
-        e = n.toElement();
-        if (!e.isNull() && e.tagName() == "GRADIENT") {
-            parseKarbonGradient(e);
-        }
-    }
-}
-
-void KoStopGradient::loadSvgGradient(QIODevice *file)
-{
-    QDomDocument doc;
-
-    if (!(doc.setContent(file)))
-        file->close();
-    else {
+    } else {
         for (QDomNode n = doc.documentElement().firstChild(); !n.isNull(); n = n.nextSibling()) {
             QDomElement e = n.toElement();
 
@@ -341,110 +353,11 @@ void KoStopGradient::loadSvgGradient(QIODevice *file)
     }
 }
 
-void KoStopGradient::parseKarbonGradient(const QDomElement& element)
-{
-    m_start = QPointF(element.attribute("originX", "0.0").toDouble(), element.attribute("originY", "0.0").toDouble());
-    m_focalPoint = QPointF(element.attribute("focalX", "0.0").toDouble(), element.attribute("focalY", "0.0").toDouble());
-    m_stop = QPointF(element.attribute("vectorX", "0.0").toDouble(), element.attribute("vectorY", "0.0").toDouble());
-
-    setType((QGradient::Type)element.attribute("type", 0).toInt());
-    setSpread((QGradient::Spread)element.attribute("repeatMethod", 0).toInt());
-
-    m_stops.clear();
-
-    qreal color1, color2, color3, color4, opacity;
-    KoColor color;
-    // load stops
-    QDomNodeList list = element.childNodes();
-    for (int i = 0; i < list.count(); ++i) {
-
-        if (list.item(i).isElement()) {
-            QDomElement colorstop = list.item(i).toElement();
-
-            if (colorstop.tagName() == "COLORSTOP") {
-                QDomElement e = colorstop.firstChild().toElement();
-
-                opacity = e.attribute("opacity", "1.0").toFloat();
-
-                QColor tmpColor;
-                const KoColorSpace* stopColorSpace;
-                switch (e.attribute("colorSpace").toUShort()) {
-                case 1:  // cmyk
-                    color1 = e.attribute("v1", "0.0").toFloat();
-                    color2 = e.attribute("v2", "0.0").toFloat();
-                    color3 = e.attribute("v3", "0.0").toFloat();
-                    color4 = e.attribute("v4", "0.0").toFloat();
-
-                    stopColorSpace = KoColorSpaceRegistry::instance()->colorSpace( CMYKAColorModelID.id(), Integer8BitsColorDepthID.id(), QString());
-                    if (stopColorSpace) {
-                        quint8 data[5];
-                        data[0] = static_cast<quint8>(color1 * 255 + 0.5);
-                        data[1] = static_cast<quint8>(color2 * 255 + 0.5);
-                        data[2] = static_cast<quint8>(color3 * 255 + 0.5);
-                        data[3] = static_cast<quint8>(color4 * 255 + 0.5);
-                        data[4] = static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5);
-                        color.setColor(data, stopColorSpace);
-                    } else {
-                        // cmyk colorspace not found fallback to rgb
-                        color.convertTo(KoColorSpaceRegistry::instance()->rgb8());
-                        tmpColor.setCmykF(color1, color2, color3, color4);
-                        tmpColor.setAlpha(static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5));
-                        color.fromQColor(tmpColor);
-                    }
-                    break;
-                case 2: // hsv
-                    color1 = e.attribute("v1", "0.0").toFloat();
-                    color2 = e.attribute("v2", "0.0").toFloat();
-                    color3 = e.attribute("v3", "0.0").toFloat();
-
-                    color.convertTo(KoColorSpaceRegistry::instance()->rgb8());
-                    tmpColor.setHsvF(color1, color2, color3);
-                    tmpColor.setAlpha(static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5));
-                    color.fromQColor(tmpColor);
-                    break;
-                case 3: // gray
-                    color1 = e.attribute("v1", "0.0").toFloat();
-                    stopColorSpace = KoColorSpaceRegistry::instance()->colorSpace( GrayAColorModelID.id(), Integer8BitsColorDepthID.id(), QString());
-                    if (stopColorSpace) {
-                        quint8 data[2];
-                        data[0] = static_cast<quint8>(color1 * 255 + 0.5);
-                        data[1] = static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5);
-                        color.setColor(data, stopColorSpace);
-                    } else {
-                        // gray colorspace not found fallback to rgb
-                        color.convertTo(KoColorSpaceRegistry::instance()->rgb8());
-                        tmpColor.setRgbF(color1, color1, color1);
-                        tmpColor.setAlpha(static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5));
-                        color.fromQColor(tmpColor);
-                    }
-                    break;
-                default: // rgb
-                    color1 = e.attribute("v1", "0.0").toFloat();
-                    color2 = e.attribute("v2", "0.0").toFloat();
-                    color3 = e.attribute("v3", "0.0").toFloat();
-                    stopColorSpace = KoColorSpaceRegistry::instance()->rgb8();
-
-                    quint8 data[4];
-                    data[2] = static_cast<quint8>(color1 * 255 + 0.5);
-                    data[1] = static_cast<quint8>(color2 * 255 + 0.5);
-                    data[0] = static_cast<quint8>(color3 * 255 + 0.5);
-                    data[3] = static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5);
-
-                    color.setColor(data, stopColorSpace);
-                }
-
-                qreal offset = colorstop.attribute("ramppoint", "0.0").toFloat();
-//              midpoint = colorstop.attribute("midpoint", "0.5").toFloat();
-
-                m_stops.append(KoGradientStop(offset, color));
-            }
-        }
-    }
-}
 
 void KoStopGradient::parseSvgGradient(const QDomElement& element)
 {
     m_stops.clear();
+    m_hasVariableStops = false;
     setSpread(QGradient::PadSpread);
 
     /*QString href = e.attribute( "xlink:href" ).mid( 1 );
@@ -492,12 +405,14 @@ void KoStopGradient::parseSvgGradient(const QDomElement& element)
 
             m_start = QPointF(xOrigin, yOrigin);
             m_stop = QPointF(xVector, yVector);
-        } else {
+        }
+        else {
             m_start = QPointF(element.attribute("x1").toDouble(), element.attribute("y1").toDouble());
             m_stop = QPointF(element.attribute("x2").toDouble(), element.attribute("y2").toDouble());
         }
         setType(QGradient::LinearGradient);
-    } else {
+    }
+    else {
         if (bbox) {
             QString s;
 
@@ -552,10 +467,11 @@ void KoStopGradient::parseSvgGradient(const QDomElement& element)
             m_start = QPointF(xOrigin, yOrigin);
             m_stop = QPointF(xVector, yVector);
             m_focalPoint = QPointF(xFocal, yFocal);
-        } else {
+        }
+        else {
             m_start = QPointF(element.attribute("cx").toDouble(), element.attribute("cy").toDouble());
             m_stop = QPointF(element.attribute("cx").toDouble() + element.attribute("r").toDouble(),
-                             element.attribute("cy").toDouble());
+                element.attribute("cy").toDouble());
             m_focalPoint = QPointF(element.attribute("fx").toDouble(), element.attribute("fy").toDouble());
         }
         setType(QGradient::RadialGradient);
@@ -579,7 +495,8 @@ void KoStopGradient::parseSvgGradient(const QDomElement& element)
             if (temp.contains('%')) {
                 temp = temp.left(temp.length() - 1);
                 off = temp.toFloat() / 100.0;
-            } else
+            }
+            else
                 off = temp.toFloat();
 
             if (!colorstop.attribute("stop-color").isEmpty())
@@ -588,7 +505,7 @@ void KoStopGradient::parseSvgGradient(const QDomElement& element)
                 // try style attr
                 QString style = colorstop.attribute("style").simplified();
                 QStringList substyles = style.split(';', QString::SkipEmptyParts);
-                Q_FOREACH (const QString & s, substyles) {
+                Q_FOREACH(const QString & s, substyles) {
                     QStringList substyle = s.split(':');
                     QString command = substyle[0].trimmed();
                     QString params = substyle[1].trimmed();
@@ -605,18 +522,22 @@ void KoStopGradient::parseSvgGradient(const QDomElement& element)
             KoColor color(rgbColorSpace);
             color.fromQColor(c);
             color.setOpacity(static_cast<quint8>(opacity * OPACITY_OPAQUE_U8 + 0.5));
-
+            QString stopTypeStr = colorstop.attribute("krita:stop-type", "color-stop");
+            KoGradientStopType stopType = KoGradientStop::typeFromString(stopTypeStr);
+            if (stopType != COLORSTOP) {
+                m_hasVariableStops = true;
+            }
             //According to the SVG spec each gradient offset has to be equal to or greater than the previous one
             //if not it needs to be adjusted to be equal
-            if (m_stops.count() > 0 && m_stops.last().first >= off) {
-                off = m_stops.last().first;
+            if (m_stops.count() > 0 && m_stops.last().position >= off) {
+                off = m_stops.last().position;
             }
-            m_stops.append(KoGradientStop(off, color));
+            m_stops.append(KoGradientStop(off, color, stopType));
         }
     }
 }
 
-void KoStopGradient::parseSvgColor(QColor &color, const QString &s)
+void KoStopGradient::parseSvgColor(QColor& color, const QString& s)
 {
     if (s.startsWith("rgb(")) {
         QString parse = s.trimmed();
@@ -641,7 +562,8 @@ void KoStopGradient::parseSvgColor(QColor &color, const QString &s)
         }
 
         color = QColor(r.toInt(), g.toInt(), b.toInt());
-    } else {
+    }
+    else {
         QString rgbColor = s.trimmed();
         QColor c;
         if (rgbColor.startsWith('#'))
@@ -658,38 +580,40 @@ QString KoStopGradient::defaultFileExtension() const
     return QString(".svg");
 }
 
-void KoStopGradient::toXML(QDomDocument &doc, QDomElement &gradientElt) const
+void KoStopGradient::toXML(QDomDocument& doc, QDomElement& gradientElt) const
 {
     gradientElt.setAttribute("type", "stop");
-    for (int s = 0; s<m_stops.size(); s++) {
+    for (int s = 0; s < m_stops.size(); s++) {
         KoGradientStop stop = m_stops.at(s);
         QDomElement stopElt = doc.createElement("stop");
-        stopElt.setAttribute("offset", stop.first);
-        stopElt.setAttribute("bitdepth", stop.second.colorSpace()->colorDepthId().id());
-        stopElt.setAttribute("alpha", stop.second.opacityF());
-        stop.second.toXML(doc, stopElt);
+        stopElt.setAttribute("offset", KisDomUtils::toString(stop.position));
+        stopElt.setAttribute("bitdepth", stop.color.colorSpace()->colorDepthId().id());
+        stopElt.setAttribute("alpha", KisDomUtils::toString(stop.color.opacityF()));
+        stopElt.setAttribute("stoptype", KisDomUtils::toString(stop.type));
+        stop.color.toXML(doc, stopElt);
         gradientElt.appendChild(stopElt);
     }
 }
 
-KoStopGradient KoStopGradient::fromXML(const QDomElement &elt)
+KoStopGradient KoStopGradient::fromXML(const QDomElement& elt)
 {
     KoStopGradient gradient;
     QList<KoGradientStop> stops;
     QDomElement stopElt = elt.firstChildElement("stop");
     while (!stopElt.isNull()) {
-        qreal offset = stopElt.attribute("offset", "0").toDouble();
+        qreal offset = KisDomUtils::toDouble(stopElt.attribute("offset", "0.0"));
         QString bitDepth = stopElt.attribute("bitdepth", Integer8BitsColorDepthID.id());
         KoColor color = KoColor::fromXML(stopElt.firstChildElement(), bitDepth);
-        color.setOpacity(stopElt.attribute("alpha", "1.0").toDouble());
-        stops.append(KoGradientStop(offset, color));
+        color.setOpacity(KisDomUtils::toDouble(stopElt.attribute("alpha", "1.0")));
+        KoGradientStopType stoptype = static_cast<KoGradientStopType>(KisDomUtils::toInt(stopElt.attribute("stoptype", "0")));
+        stops.append(KoGradientStop(offset, color, stoptype));
         stopElt = stopElt.nextSiblingElement("stop");
     }
     gradient.setStops(stops);
     return gradient;
 }
 
-bool KoStopGradient::saveToDevice(QIODevice *dev) const
+bool KoStopGradient::saveToDevice(QIODevice* dev) const
 {
     QTextStream stream(dev);
 
@@ -701,7 +625,10 @@ bool KoStopGradient::saveToDevice(QIODevice *dev) const
 
     const QString indent = "    ";
 
-    stream << "<svg>" << endl;
+    stream << "<svg xmlns=\"http://www.w3.org/2000/svg\" \n";
+    stream << QString("    xmlns:krita=\"%1\"\n").arg(KoXmlNS::krita);
+    stream << ">" << endl;
+
 
     stream << indent;
     stream << "<linearGradient id=\"" << name() << "\" ";
@@ -712,13 +639,16 @@ bool KoStopGradient::saveToDevice(QIODevice *dev) const
     QColor color;
 
     // color stops
-    Q_FOREACH (const KoGradientStop & stop, m_stops) {
-        stop.second.toQColor(&color);
+    Q_FOREACH(const KoGradientStop & stop, m_stops) {
+        stop.color.toQColor(&color);
         stream << indent << indent;
         stream << "<stop stop-color=\"";
         stream << color.name();
-        stream << "\" offset=\"" << QString().setNum(stop.first);
-        stream << "\" stop-opacity=\"" << static_cast<float>(color.alpha()) / 255.0f << "\"" << " />" << endl;
+        stream << "\" offset=\"" << QString().setNum(stop.position);
+        stream << "\" stop-opacity=\"" << static_cast<float>(color.alpha()) / 255.0f;
+        stream << "\" krita:stop-type=\"" << stop.typeString() << "\"";
+            
+        stream << " />" << endl;
     }
     stream << indent;
     stream << "</linearGradient>" << endl;
@@ -727,4 +657,4 @@ bool KoStopGradient::saveToDevice(QIODevice *dev) const
     KoResource::saveToDevice(dev);
 
     return true;
-}
+}
\ No newline at end of file
