@@ -77,7 +77,7 @@ public:
      * @param viewConverter the viewconverter for converting between
      *                       window and document coordinates.
      */
-    KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceManager *resourceManager, KisView *view, KoShapeBasedDocumentBase *sc);
+    KisCanvas2(KisCoordinatesConverter *coordConverter, KoCanvasResourceProvider *resourceManager, KisMainWindow *mainWindow, KisView *view, KoShapeControllerBase *sc);
 
     ~KisCanvas2() override;
 
@@ -120,6 +120,13 @@ public: // KoCanvasBase implementation
      */
     KoShapeManager *globalShapeManager() const;
 
+    /**
+     * Return shape manager associated with the currently active node.
+     * If current node has no internal shape manager, return null.
+     */
+    KoShapeManager *localShapeManager() const;
+
+
     void updateCanvas(const QRectF& rc) override;
 
     void updateInputMethodInfo() override;
@@ -215,7 +222,7 @@ public: // KisCanvas2 methods
     QRect regionOfInterest() const;
 
     /**
-     * Set aftificial limit outside which the image will not be rendered
+     * Set artificial limit outside which the image will not be rendered
      * \p rc is measured in image pixels
      */
     void setRenderingLimit(const QRect &rc);
@@ -258,10 +265,6 @@ public Q_SLOTS:
 
     void channelSelectionChanged();
 
-    /**
-     * Called whenever the display monitor profile resource changes
-     */
-    void slotSetDisplayProfile(const KoColorProfile *profile);
     void startUpdateInPatches(const QRect &imageRect);
 
     void slotTrySwitchShapeManager();
@@ -279,6 +282,9 @@ private Q_SLOTS:
     void startUpdateCanvasProjection(const QRect & rc);
     void updateCanvasProjection();
 
+    void slotBeginUpdatesBatch();
+    void slotEndUpdatesBatch();
+    void slotSetLodUpdatesBlocked(bool value);
 
     /**
      * Called whenever the view widget needs to show a different part of
@@ -298,6 +304,7 @@ private Q_SLOTS:
 
     void slotReferenceImagesChanged();
 
+    void slotImageColorSpaceChanged();
 public:
 
     bool isPopupPaletteVisible() const;
@@ -311,6 +318,7 @@ public:
     bool lodAllowedInCanvas() const;
 
     void initializeImage();
+    void disconnectImage();
 
     void setFavoriteResourceManager(KisFavoriteResourceManager* favoriteResourceManager);
 
@@ -324,6 +332,7 @@ private:
     void updateCanvasWidgetImpl(const QRect &rc = QRect());
     void setCanvasWidget(KisAbstractCanvasWidget *widget);
     void resetCanvas(bool useOpenGL);
+    void setDisplayProfile(const KoColorProfile *profile);
 
     void notifyLevelOfDetailChange();
 
