@@ -24,12 +24,14 @@
 #include <kconfig.h>
 #include <kconfiggroup.h>
 
+#include <kis_painter.h>
+
 #include "kis_tool_paint.h"
-#include "kis_painter.h"
+#include "KisSelectionToolFactoryBase.h"
+#include "KisToolShapeUtils.h"
+
 #include "ui_wdggeometryoptions.h"
 
-
-
 class KoCanvasBase;
 class KoPathShape;
 
@@ -62,15 +64,27 @@ public Q_SLOTS:
     void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes) override;
     virtual void outlineSettingChanged(int value);
     virtual void fillSettingChanged(int value);
+    virtual void patternRotationSettingChanged(qreal value);
+    virtual void patternScaleSettingChanged(qreal value);
 
 protected:
     QWidget* createOptionWidget() override;
 
-    virtual KisPainter::FillStyle fillStyle();
-    KisPainter::StrokeStyle strokeStyle();
+    KisToolShapeUtils::FillStyle fillStyle();
+    KisToolShapeUtils::StrokeStyle strokeStyle();
+    QTransform fillTransform();
 
     qreal currentStrokeWidth() const;
 
+    struct KRITAUI_EXPORT ShapeAddInfo {
+        bool shouldAddShape = false;
+        bool shouldAddSelectionShape = false;
+
+        void markAsSelectionShapeIfNeeded(KoShape *shape) const;
+    };
+
+    ShapeAddInfo shouldAddShape(KisNodeSP currentNode) const;
+
     void addShape(KoShape* shape);
 
     void addPathShape(KoPathShape* pathShape, const KUndo2MagicString& name);
