@@ -38,11 +38,13 @@ KisCmbIDList::~KisCmbIDList()
 }
 
 
-void KisCmbIDList::setIDList(const QList<KoID>  & list)
+void KisCmbIDList::setIDList(const QList<KoID>  & list, bool sorted)
 {
     clear();
     m_list = list;
-    std::sort(m_list.begin(), m_list.end(), KoID::compareNames);
+    if (sorted) {
+        std::sort(m_list.begin(), m_list.end(), KoID::compareNames);
+    }
     for (qint32 i = 0; i < m_list.count(); ++i) {
         addItem(m_list.at(i).name());
     }
