@@ -54,8 +54,15 @@ public:
 
     /// For paragraphs each tab definition is represented by this struct.
     struct Tab {
-        qreal position;    ///< distance in point from the start of the text-shape
-        QTextOption::TabType type;       ///< Determine which type is used.
+        qreal position { 0.0 };    ///< distance in point from the start of the text-shape
+        QTextOption::TabType type { QTextOption::LeftTab };       ///< Determine which type is used.
+
+        Tab() {}
+        Tab(qreal _position, QTextOption::TabType _type)
+        {
+            position = _position;
+            type = _type;
+        }
     };
 
     /// The ruler's unit
@@ -194,14 +201,14 @@ public Q_SLOTS:
 
     /**
      * Set whether the tabs is relative to the paragraph indent
-     * @param relative tabs are relative to pragraph indent if true, default is false.
+     * @param relative tabs are relative to paragraph indent if true, default is false.
      */
     void setRelativeTabs(bool relative);
 
     /**
      * Update the tabs
      * @param tabs a list of tabs that is shown on the ruler
-     * @param tabDistance the distncte between regular interval tabs
+     * @param tabDistance the distance between regular interval tabs
      */
     void updateTabs(const QList<Tab> &tabs, qreal tabDistance);
 
