@@ -3,7 +3,8 @@
  *
  *  This library is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2.1 of the License.
+ *  the Free Software Foundation; version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,11 +29,11 @@ class KisColorHistory : public KisColorPatches
     Q_OBJECT
 public:
     explicit KisColorHistory(QWidget *parent = 0);
-    void setCanvas(KisCanvas2 *canvas);
-    void unsetCanvas();
+    void setCanvas(KisCanvas2 *canvas) override;
+    void unsetCanvas() override;
 
 protected:
-    KisColorSelectorBase* createPopup() const;
+    KisColorSelectorBase* createPopup() const override;
 
 public Q_SLOTS:
     void addColorToHistory(const KoColor& color);
