@@ -30,6 +30,7 @@
 #include <kis_debug.h>
 #include <kpluginfactory.h>
 
+#include <filter/kis_filter_category_ids.h>
 #include <filter/kis_filter_registry.h>
 #include <kis_global.h>
 #include "KisGradientSlider.h"
@@ -61,7 +62,7 @@ KritaThreshold::~KritaThreshold()
 }
 
 KisFilterThreshold::KisFilterThreshold()
-    : KisFilter(id(), categoryAdjust(), i18n("&Threshold..."))
+    : KisFilter(id(), FiltersCategoryAdjustId, i18n("&Threshold..."))
 {
     setColorSpaceIndependence(FULLY_INDEPENDENT);
 
@@ -101,14 +102,14 @@ void KisFilterThreshold::processImpl(KisPaintDeviceSP device,
 }
 
 
-KisFilterConfigurationSP KisFilterThreshold::factoryConfiguration() const
+KisFilterConfigurationSP KisFilterThreshold::defaultConfiguration() const
 {
-    KisFilterConfigurationSP config = new KisFilterConfiguration("threshold", 1);
+    KisFilterConfigurationSP config = factoryConfiguration();
     config->setProperty("threshold", 128);
     return config;
 }
 
-KisConfigWidget *KisFilterThreshold::createConfigurationWidget(QWidget *parent, const KisPaintDeviceSP dev) const
+KisConfigWidget *KisFilterThreshold::createConfigurationWidget(QWidget *parent, const KisPaintDeviceSP dev, bool) const
 {
     return new KisThresholdConfigWidget(parent, dev);
 }
