@@ -55,9 +55,9 @@ KisBrushImport::~KisBrushImport()
 }
 
 
-KisImportExportFilter::ConversionStatus KisBrushImport::convert(KisDocument *document, QIODevice *io, KisPropertiesConfigurationSP /*configuration*/)
+KisImportExportErrorCode KisBrushImport::convert(KisDocument *document, QIODevice *io, KisPropertiesConfigurationSP /*configuration*/)
 {
-    KisBrush *brush;
+    KisColorfulBrush *brush;
 
     if (mimeType() == "image/x-gimp-brush") {
         brush = new KisGbrBrush(filename());
@@ -66,21 +66,21 @@ KisImportExportFilter::ConversionStatus KisBrushImport::convert(KisDocument *doc
         brush = new KisImagePipeBrush(filename());
     }
     else {
-        return KisImportExportFilter::BadMimeType;
+        return ImportExportCodes::FileFormatIncorrect;
     }
 
     if (!brush->loadFromDevice(io)) {
         delete brush;
-        return KisImportExportFilter::InvalidFormat;
+        return ImportExportCodes::FileFormatIncorrect;
     }
 
     if (!brush->valid()) {
         delete brush;
-        return KisImportExportFilter::InvalidFormat;
+        return ImportExportCodes::FileFormatIncorrect;;
     }
 
     const KoColorSpace *colorSpace = 0;
-    if (brush->hasColor()) {
+    if (brush->isImageType()) {
         colorSpace = KoColorSpaceRegistry::instance()->rgb8();
     }
     else {
@@ -96,7 +96,7 @@ KisImportExportFilter::ConversionStatus KisBrushImport::convert(KisDocument *doc
         for(int i = brushes.size(); i > 0; i--) {
             KisGbrBrush *subbrush = brushes.at(i - 1);
             const KoColorSpace *subColorSpace = 0;
-            if (brush->hasColor()) {
+            if (brush->isImageType()) {
                 subColorSpace = KoColorSpaceRegistry::instance()->rgb8();
             }
             else {
@@ -117,7 +117,7 @@ KisImportExportFilter::ConversionStatus KisBrushImport::convert(KisDocument *doc
 
     document->setCurrentImage(image);
     delete brush;
-    return KisImportExportFilter::OK;
+    return ImportExportCodes::OK;
 
 }
 #include "kis_brush_import.moc"
