@@ -22,6 +22,8 @@
 
 #include <cmath>
 
+#include <QPainterPath>
+
 #include <KoCompositeOpRegistry.h>
 
 #include <kis_debug.h>
@@ -71,9 +73,6 @@ KisExperimentPaintOp::KisExperimentPaintOp(const KisPaintOpSettingsSP settings,
         m_originalPainter->setPattern(painter->pattern());        
         m_originalPainter->setFillStyle(m_fillStyle);
 
-
-
-
     }
     else {
         m_originalPainter = 0;
@@ -85,7 +84,7 @@ KisExperimentPaintOp::~KisExperimentPaintOp()
     delete m_originalPainter;
 }
 
-void KisExperimentPaintOp::paintRegion(const QRegion &changedRegion)
+void KisExperimentPaintOp::paintRegion(const KisRegion &changedRegion)
 {
     if (m_windingFill) {
         m_path.setFillRule(Qt::WindingFill);
@@ -228,7 +227,7 @@ void KisExperimentPaintOp::paintLine(const KisPaintInformation &pi1, const KisPa
                  */
                 const int pathSizeThreshold = 128;
 
-                QRegion changedRegion;
+                KisRegion changedRegion;
                 if (distanceMetric < pathSizeThreshold) {
 
                     QRectF changedRect = m_path.boundingRect().toRect() |
@@ -248,7 +247,7 @@ void KisExperimentPaintOp::paintLine(const KisPaintInformation &pi1, const KisPa
                 m_lastPaintedPath = m_path;
             }
             else if (!m_savedPoints.isEmpty()) {
-                QRegion changedRegion = KritaUtils::splitTriangles(m_center, m_savedPoints);
+                KisRegion changedRegion = KritaUtils::splitTriangles(m_center, m_savedPoints);
                 paintRegion(changedRegion);
             }
 
